﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.Diagnostics;

namespace BackgroundWorkerDemo
{
    public partial class Form1 : Form
    {
        private readonly BackgroundWorker _worker = new BackgroundWorker() { WorkerReportsProgress = true };
        // true means initial state is "has singal".
        // this is a global value to indicate the current state, can be changed when calling _manualReset.Set or _manualReset.Reset.
        private readonly ManualResetEvent _manualReset = new ManualResetEvent(true);

        public Form1()
        {
            InitializeComponent();
            _worker.DoWork += new DoWorkEventHandler(_worker_DoWork);
            _worker.ProgressChanged += new ProgressChangedEventHandler(_worker_ProgressChanged);
            _worker.RunWorkerCompleted += new RunWorkerCompletedEventHandler(_worker_RunWorkerCompleted);

            //new Test().BT_Temp_Click(); // another test.
        }

        void _worker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            btnStart.Text = "Start";
            MessageBox.Show("All finished.");
        }

        void _worker_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            progressBar1.Value = e.ProgressPercentage;
            label1.Text = e.ProgressPercentage + "%";
        }

        void _worker_DoWork(object sender, DoWorkEventArgs e)
        {
            DoWork();
        }

        private void DoWork()
        {
            for (int i = 0; i < 10000; i++)
            {
                Thread.Sleep(200);
                Debug.WriteLine(i);
                _worker.ReportProgress(i);
                // if current state is "has singal", do not block the "DoWork" thread. Note: code in "DoWork" is in a new thread.
                // if current state is "no singal", block the "DoWork" thread, unitl current state is changed to "has singal"(by calling _manualReset.Set()).
                _manualReset.WaitOne();
            }
        }

        private void btnStart_Click(object sender, EventArgs e)
        {
            if (btnStart.Text == "Start")
            {
                this.Start();
            }
            else if (btnStart.Text == "Pause")
            {
                this.Pause();
            }
            else
            {
                this.Resume();
            }
        }

        private void Start()
        {
            if (!_worker.IsBusy)
            {
                btnStart.Text = "Pause";
                _worker.RunWorkerAsync();
            }
        }

        private void Pause()
        {
            _manualReset.Reset();
            btnStart.Text = "Continue";
        }

        private void Resume()
        {
            _manualReset.Set();
            btnStart.Text = "Pause";
        }
    }

    //another test
    public class Test
    {
        // if initial state is false, no messagebox shows.
        // otherwise, both will show.
        ManualResetEvent _manualResetEvent = new ManualResetEvent(true);

        public void BT_Temp_Click()
        {
            Thread t1 = new Thread(this.Thread1Foo);
            t1.Start();
            Thread t2 = new Thread(this.Thread2Foo);
            t2.Start();
        }

        void Thread1Foo()
        {
            _manualResetEvent.WaitOne();

            MessageBox.Show("t1 end");
        }

        void Thread2Foo()
        {
            _manualResetEvent.WaitOne();

            MessageBox.Show("t2 end");
        }
    }
}
